package com.persagy.dmp.rwd.digital.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.ReflectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSONArray;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.node.JsonNodeFactory;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.dmp.common.constant.ValidEnum;
import com.persagy.dmp.common.context.AppContext;
import com.persagy.dmp.common.model.entity.BaseEntity;
import com.persagy.dmp.common.model.response.CommonResult;
import com.persagy.dmp.common.utils.ResultHelper;
import com.persagy.dmp.define.entity.ObjectInfoDefine;
import com.persagy.dmp.define.entity.ObjectTypeDefine;
import com.persagy.dmp.digital.entity.ObjectDigital;
import com.persagy.dmp.rwd.basic.constant.DigitalMessageConstant;
import com.persagy.dmp.rwd.basic.constant.DigitalObjectType;
import com.persagy.dmp.rwd.basic.constant.DigitalRelCode;
import com.persagy.dmp.rwd.basic.dto.RequestData;
import com.persagy.dmp.rwd.basic.utils.DigitalCommonUtils;
import com.persagy.dmp.rwd.basic.utils.DigitalMessageHelper;
import com.persagy.dmp.rwd.define.service.IObjectInfoService;
import com.persagy.dmp.rwd.define.service.IObjectTypeService;
import com.persagy.dmp.rwd.digital.dao.ObjectDigitalMapper;
import com.persagy.dmp.rwd.digital.dao.ObjectRelationMapper;
import com.persagy.dmp.rwd.digital.service.IObjectDigitalService;
import com.persagy.dmp.rwd.parser.service.InfoDataFactory;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 对象数据 实现类
 * @author Charlie Yu
 * @date 2021-07-08
 */
@Service
@Transactional(rollbackFor = Exception.class)
@RequiredArgsConstructor
public class ObjectDigitalServiceImpl implements IObjectDigitalService {

    private final ObjectDigitalMapper dao;
    private final DigitalMessageHelper messageSender;
    private final IObjectInfoService infoService;
    private final IObjectTypeService typeService;
    private final ObjectDigitalMapper objectDigitalMapper;
    private final ObjectRelationMapper objectRelationMapper;

    @Override
    public ObjectDigital load(String id) {
        return dao.selectById(id);
    }

    @Override
    public List<ObjectDigital> queryByCondition(Wrapper<ObjectDigital> queryWrapper) {
        return dao.selectList(queryWrapper);
    }

    @Override
    public Integer queryCount(Wrapper<ObjectDigital> queryWrapper) {
        return dao.selectCount(queryWrapper);
    }

    @Override
    public Page queryByCondition(Page page, Wrapper<ObjectDigital> queryWrapper) {
        return dao.selectPage(page, queryWrapper);
    }

    @Override
    public List<ObjectDigital> insert(List<ObjectDigital> voList) {
        if(CollUtil.isEmpty(voList)) {
            return null;
        }
        validateOnSave(voList, false);
        //获取对象类型
        Map<String, String> typeMap = typeService.queryObjTypeMap();
        voList.forEach(vo -> {
            // 生成ID
            if (StrUtil.isBlank(vo.getId())){
                if (StrUtil.isNotBlank(vo.getObjType())){
                    vo.setId(DigitalCommonUtils.fetchObjectId(vo.getProjectId(), vo.getObjType()));
                }else {
                    vo.setId(DigitalCommonUtils.fetchObjectId(vo.getProjectId(), typeMap.get(vo.getClassCode())));
                }
            }
            //设置对象类型-objType
            vo.setObjType(typeMap.get(vo.getClassCode()));
            vo.setValid(ValidEnum.TRUE.getType());
            // 新增
            dao.insert(vo);
            // 新增后消息
            messageSender.sendMessage(DigitalMessageConstant.OPERATE_AFTER_INSERT, null, vo, false);
        });
        return voList;
    }


    @Override
    public List<ObjectDigital> update(List<ObjectDigital> voList){
        if(CollUtil.isEmpty(voList)) {
            return null;
        }
        //获取$remove字段
        Map<String, Object> removeMap = getRemoveInfo(voList);
        //获取基础属性
        Set<String> fieldSet = ReflectUtil.getFieldMap(ObjectDigital.class).keySet();
        //查询数据库中存在的数据
        List<String> idList = CollUtil.getFieldValues(voList, BaseEntity.PROP_ID, String.class);
        List<ObjectDigital> dbList = dao.selectBatchIds(idList);
        Map<String, ObjectDigital> dbMap = CollectionUtil.fieldValueMap(dbList, BaseEntity.PROP_ID);

        //填充classCode
        fillClassCodeForUpdate(voList,dbMap);
        //信息点校验
        Map<String, Map<String, ObjectInfoDefine>> allInfoMap = validateOnSave(voList, true);
        List<ObjectDigital> resultList = new ArrayList<>();
        voList.forEach(vo -> {
            ObjectDigital dbVO = MapUtil.get(dbMap, vo.getId(), ObjectDigital.class);
            // 容错处理
            if(dbVO == null) {
                return;
            }

            //数据合并
            ObjectDigital newDbVO = dbVO;
            Map<String, ObjectInfoDefine> infoDefineMap = allInfoMap.get(vo.getClassCode());
            dataMerge(newDbVO,vo,infoDefineMap,fieldSet);

            //移除需要删除的信息点
            Object remove = removeMap.get(newDbVO.getId());
            remove(newDbVO,remove,fieldSet);

            // 修改
            dao.updateById(newDbVO);
            resultList.add(newDbVO);
            // 修改后消息
            messageSender.sendMessage(DigitalMessageConstant.OPERATE_AFTER_UPDATE, dbVO, newDbVO, false);
        });
        return resultList;
    }

    @Override
    public void delete(List<String> idList) {
        if(CollUtil.isEmpty(idList)) {
            return;
        }
        List<ObjectDigital> dbList = dao.selectBatchIds(idList);
        Map<String, ObjectDigital> dbMap = CollectionUtil.fieldValueMap(dbList, BaseEntity.PROP_ID);
        idList.forEach(id -> {
            ObjectDigital dbVO = MapUtil.get(dbMap, id, ObjectDigital.class);
            // 容错处理
            if(dbVO == null) {
                return;
            }
            // 删除
            ObjectDigital vo = new ObjectDigital();
            vo.setId(id);
            vo.setValid(ValidEnum.FALSE.getType());
            dao.updateById(vo);
            // 删除后消息
            dbVO.setValid(ValidEnum.FALSE.getType());
            messageSender.sendMessage(DigitalMessageConstant.OPERATE_AFTER_DELETE, null, dbVO, false);
        });
    }

    /***
     * Description: 根据上级id查询下级关联对象
     * @param requestData : 请求参数
     * @return : com.persagy.dmp.common.model.response.CommonResult<java.util.List<com.persagy.dmp.digital.entity.ObjectDigital>>
     * @author : lijie
     * @date :2021/8/24 16:37
     * Update By lijie 2021/8/24 16:37
     */
    @Override
    public CommonResult<List<ObjectDigital>> queryObjectListSuperiorId(RequestData requestData) {
        // 1.计算分页参数
        Long page = requestData.getPage();
        Long size = requestData.getSize();
        // 防止人不自觉,最大仅允许查500条数据
        if (null==page){
            page=1L;
        }
        if (null==size){
            size=500L;
            requestData.setSize(500L);
        }
        requestData.setSkip((page-1)*size);
        // 2.当只有项目id时不查询关系,直接根据项目id筛选所有对象
        if (StrUtil.isBlank(requestData.getBuildingId())
                && StrUtil.isBlank(requestData.getFloorId())
                && StrUtil.isBlank(requestData.getEquipmentId())){
            return queryObjectListByProjectId(requestData);
        }
        // 3.存在建筑id/楼层id/设备id
        // 3.1 获取关系code列表
        fillSubordinateRelCode(requestData);
        // 3.2 调用查询语句查询相关内容
        return analysisQueryResultList(objectDigitalMapper.queryObjectListBySuperiorId(requestData),requestData,false);
    }
    /***
     * Description: 根据对象id获取下级关系code数组
     * @param requestData : 请求参数
     * @return : void
     * @author : lijie
     * @date :2021/8/24 21:20
     * Update By lijie 2021/8/24 21:20
     */
    private void fillSubordinateRelCode(RequestData requestData) {
        if (StrUtil.isNotBlank(requestData.getEquipmentId())){
            // 设备下的部件
            requestData.setRelCodes(CollUtil.newHashSet(DigitalRelCode.EQ_2_EC.getIndex()));
            requestData.setFilterObjId(requestData.getEquipmentId());
            return;
        }
        if (StrUtil.isNotBlank(requestData.getFloorId())){
            // 楼层下的空间,设备,系统
            requestData.setRelCodes(CollUtil.newHashSet(DigitalRelCode.FL_2_SP.getIndex(),
                    DigitalRelCode.EQ_2_FL.getIndex(),DigitalRelCode.SY_2_FL.getIndex()));
            // 设置设备查询部件
            requestData.setQueryEcFlag(true);
            requestData.setEcRelCodes(CollUtil.newHashSet(DigitalRelCode.EQ_2_EC.getIndex()));
            requestData.setFilterObjId(requestData.getFloorId());
            return;
        }
        if (StrUtil.isNotBlank(requestData.getBuildingId())){
            // 建筑下的楼层,空间,设备,系统
            requestData.setRelCodes(CollUtil.newHashSet(DigitalRelCode.BD_2_FL.getIndex(),
                    DigitalRelCode.BD_2_SP.getIndex(),DigitalRelCode.SY_2_BD.getIndex(),
                    DigitalRelCode.EQ_2_BD.getIndex()));
            // 设置设备查询部件
            requestData.setQueryEcFlag(true);
            requestData.setEcRelCodes(CollUtil.newHashSet(DigitalRelCode.EQ_2_EC.getIndex()));
            requestData.setFilterObjId(requestData.getBuildingId());
        }
    }

    /***
     * Description: 根据项目id查询对象列表
     * @param requestData : 请求参数
     * @return : com.persagy.dmp.common.model.response.CommonResult<java.util.List<com.persagy.dmp.digital.entity.ObjectDigital>>
     * @author : lijie
     * @date :2021/8/24 16:40
     * Update By lijie 2021/8/24 16:40
     */
    private CommonResult<List<ObjectDigital>> queryObjectListByProjectId(RequestData requestData) {
        // 1.查询结果集
        List<List<ObjectDigital>> resultList =
                objectDigitalMapper.queryObjectDigitalsByProjectId(requestData,requestData.getSkip(),requestData.getSize());
        return analysisQueryResultList(resultList,requestData,true);
    }
    /***
     * Description: 解析结果集
     * @param resultList : 结果集
     * @param requestData : 请求参数
     * @return : com.persagy.dmp.common.model.response.CommonResult<java.util.List<com.persagy.dmp.digital.entity.ObjectDigital>>
     * @author : lijie
     * @date :2021/8/25 13:16
     * Update By lijie 2021/8/25 13:16
     */
    private CommonResult<List<ObjectDigital>> analysisQueryResultList(List<List<ObjectDigital>> resultList,
                                                                      RequestData requestData,
                                                                      Boolean fillRelFlag) {
        if (CollUtil.isEmpty(resultList)){
            return ResultHelper.multi(new ArrayList<>(),0);
        }
        List<ObjectDigital> pageResult = resultList.get(0);
        List<ObjectDigital> totalCount = resultList.get(1);
        if (CollUtil.isEmpty(pageResult)){
            return ResultHelper.multi(new ArrayList<>(),0);
        }
        if (requestData.isOnlyCount()){
            // 只返回数
            return ResultHelper.multi(new ArrayList<>(),totalCount.get(0).getTotalCount());
        }
        fillRelIds(pageResult,fillRelFlag);
        // 2.过滤信息点
        retainObjInfos(pageResult,requestData.getReturnInfos());
        // 3.处理Fl2Sp的关系,将对应的空间对象的楼层id扔进去
        fillFloorIdIntoSpaceObjFromIds(pageResult);
        return ResultHelper.multi(pageResult,totalCount.get(0).getTotalCount());
    }

    private void fillRelIds(List<ObjectDigital> pageResult,
                            Boolean fillRelFlag) {
        if (null==fillRelFlag || !fillRelFlag){
            return;
        }
        // 注入关系
        // 2.1 根据楼层id查询建筑id
        // 2.2 根据空间id查询建筑id
        // 2.3 根据设备id查询建筑id
        // 2.4 根据系统id查询建筑id
        Set<String> objIds = pageResult.stream()
                .filter(object->!DigitalObjectType.BUILDING.getIndex().equals(object.getObjType()))
                .map(ObjectDigital::getId)
                .collect(Collectors.toSet());
        if (CollUtil.isEmpty(objIds)){
            return;
        }
        List<ObjectDigital> objectDigitals = objectRelationMapper.selectRelCodesByObjIdsAndRelCodes(objIds, CollUtil
                .newHashSet(DigitalRelCode.EQ_2_EC.getIndex(), DigitalRelCode.BD_2_FL.getIndex(),
                        DigitalRelCode.BD_2_SP.getIndex(), DigitalRelCode.SY_2_BD.getIndex(),
                        DigitalRelCode.EQ_2_BD.getIndex()));
        if (CollUtil.isEmpty(objectDigitals)){
            return;
        }
        Map<String, ObjectDigital> digitalMap = objectDigitals.stream()
                .collect(Collectors.toMap(ObjectDigital::getId, objectDigital -> objectDigital, (k1, k2) -> k1));
        for (ObjectDigital objectDigital : pageResult) {
            if (!digitalMap.containsKey(objectDigital.getId())){
                continue;
            }
            objectDigital.setObjToIds(CollUtil
                    .defaultIfEmpty(digitalMap.get(objectDigital.getId()).getObjToIds(),new HashSet<>()));
            objectDigital.setObjFromIds(CollUtil
                    .defaultIfEmpty(digitalMap.get(objectDigital.getId()).getObjFromIds(),new HashSet<>()));
        }
    }

    /***
     * Description: 填充关系id
     * @param objectDigitals : 查询结果集
     * @return : void
     * @author : lijie
     * @date :2021/8/26 12:00
     * Update By lijie 2021/8/26 12:00
     */
    private void fillFloorIdIntoSpaceObjFromIds(List<ObjectDigital> objectDigitals) {
        // 1.获得空间id
        Set<String> spaceIds = objectDigitals.stream()
                .filter(objectDigital -> DigitalObjectType.SPACE.getIndex().equals(objectDigital.getObjType()))
                .map(ObjectDigital::getId)
                .collect(Collectors.toSet());
        if (CollUtil.isEmpty(spaceIds)){
            return;
        }
        List<ObjectDigital> relResult = objectRelationMapper.selectRelCodesByObjToIdsAndRelCode(spaceIds,DigitalRelCode.FL_2_SP.getIndex());
        if (CollUtil.isEmpty(relResult)){
            return;
        }
        Map<String, Set<String>> setMap = relResult
                .stream()
                .collect(Collectors.toMap(ObjectDigital::getId, ObjectDigital::getObjFromIds, (k1, k2) -> k1));
        for (ObjectDigital objectDigital : objectDigitals) {
            Set<String> objFromIds = CollUtil.defaultIfEmpty(objectDigital.getObjFromIds(),new HashSet<>());
            objFromIds.addAll(setMap.getOrDefault(objectDigital.getId(),new HashSet<>()));
            objectDigital.setObjFromIds(objFromIds);
        }
    }

    /***
     * Description: 过滤掉不需要的信息点
     * @param objectDigitals : 查询结果集
     * @param returnInfos : 返回的信息点key
     * @return : void
     * @author : lijie
     * @date :2021/8/25 13:13
     * Update By lijie 2021/8/25 13:13
     */
    private void retainObjInfos(List<ObjectDigital> objectDigitals, Set<String> returnInfos) {
        if (CollUtil.isEmpty(returnInfos) || CollUtil.isEmpty(objectDigitals)){
            return;
        }
        for (ObjectDigital objectDigital : objectDigitals) {
            ObjectNode infos = objectDigital.getInfos();
            if (null==infos || infos.size()<=0){
                continue;
            }
            infos.retain(returnInfos);
        }
    }

    /**
     * 保存校验
     * 编码全局唯一
     * @param voList
     * @param isUpdate
     */
    private Map<String, Map<String, ObjectInfoDefine>> validateOnSave(List<ObjectDigital> voList, boolean isUpdate) {
        return validateInfoCode(voList);
    }

    /**
     * 信息点值校验
     * @param voList
     */
    private Map<String, Map<String, ObjectInfoDefine>> validateInfoCode(List<ObjectDigital> voList) {
        //获取该信息点当前classCode
        List<String> typeCodeList = CollUtil.getFieldValues(voList, "classCode", String.class);

        //查询通用类型：equipment,building,floor,space等
        Map<String, String> objTypeMap = typeService.queryObjTypeMap();
        queryParentClassCode(typeCodeList,objTypeMap);

        // 查询信息点定义
        Map<String, Map<String, ObjectInfoDefine>> allInfoMap = infoService.queryByTypes(typeCodeList);
        // 信息点校验器
        for(ObjectDigital vo:voList) {
            Map<String, ObjectInfoDefine> infoMap = allInfoMap.get(vo.getClassCode());
            String objType = objTypeMap.get(vo.getClassCode());
            //设置通用信息点
            if(!objType.equals(vo.getClassCode())){
                Map<String, ObjectInfoDefine> commonInfoDefineMap = allInfoMap.get(objType);
                if(commonInfoDefineMap!=null || commonInfoDefineMap.size()>0){
                    infoMap.putAll(commonInfoDefineMap);
                }
            }
            if(MapUtil.isEmpty(infoMap)) {
                continue;
            }
            // 信息点值对象
            ObjectNode infos = vo.getInfos();
            if(infos == null || infos.isNull()) {
                continue;
            }
            // 按infos属性循环处理
            Iterator<String> infoKeys = infos.fieldNames();
            while(infoKeys.hasNext()) {
                String infoCode = infoKeys.next();
                // 如果信息点中不存在，移除
                if(!infoMap.containsKey(infoCode)) {
                    // infos.remove(infoCode);
                    infoKeys.remove();
                    continue;
                }
                // 校验信息点值
                InfoDataFactory.parse(infoMap.get(infoCode), infos.get(infoCode));

            }
        }
        return allInfoMap;
    }

    /**
     * 获取remove信息点
     * @param voList
     * @return
     */
    private Map<String, Object> getRemoveInfo(List<ObjectDigital> voList){
        Map<String, Object> removeMap = new HashMap<>();
        for (ObjectDigital digital : voList) {
            ObjectNode infos = digital.getInfos();
            if(infos!=null){
                removeMap.put(digital.getId(),infos.get("$remove"));
            }
        }
        return removeMap;
    }

    /**
     * 查询对应的通用类型
     * @param typeCodeList
     */
    private void  queryParentClassCode(List<String> typeCodeList,Map<String, String> objTypeMap){
        if(typeCodeList == null || typeCodeList.size()<=0){
            return ;
        }
        //查询classCode对应点objType
        List<String> parentCodes = new ArrayList<>();
        for (String classCode : typeCodeList) {
            String objType = objTypeMap.get(classCode);
            if(StringUtils.isEmpty(objType) || "0".equals(objType) || parentCodes.contains(objType)){
                return ;
            }
            parentCodes.add(objType);
        }

        if(parentCodes!=null && parentCodes.size()>0){
            typeCodeList.addAll(parentCodes);
        }
        return ;
    }
    /**
     * 数据合并
     * @param dbVO 源数据
     * @param vo 目标数据
     * @param vo 待删除待数据
     */
    private void dataMerge(ObjectDigital dbVO,ObjectDigital vo,Map<String, ObjectInfoDefine> infoDefineMap,Set<String> fieldSet){
        //基础属性处理
        handleBasicColumn(dbVO,vo);
        //动态信息点处理
        ObjectNode infos = vo.getInfos();
        for (Map.Entry<String, ObjectInfoDefine> entry : infoDefineMap.entrySet()) {
            String code = entry.getKey();
            if(fieldSet.contains(code)){
                //属于基础信息点
                continue;
            }
            if (!infos.has(code)) {
                continue;
            }
            JsonNode value = infos.get(code);
            if (dbVO.getInfos() == null) {
                dbVO.setInfos(JsonNodeFactory.instance.objectNode());
            }
            dbVO.getInfos().set(code, value);
        }
    }

    /**
     * 基础属性处理
     * @param dbVO
     * @param vo
     */
    private void handleBasicColumn(ObjectDigital dbVO,ObjectDigital vo){
        if(StringUtils.isNotEmpty(vo.getName())){
            dbVO.setName(vo.getName());
        }
        if(StringUtils.isNotEmpty(vo.getLocalId())){
            dbVO.setLocalId(vo.getLocalId());
        }
        if(StringUtils.isNotEmpty(vo.getLocalName())){
            dbVO.setLocalName(vo.getLocalName());
        }
        if(vo.getVirtualCodes()!=null){
            dbVO.setVirtualCodes(vo.getVirtualCodes());
        }
        if(vo.getValid()!=null){
            dbVO.setValid(vo.getValid());
        }
    }


    /**
     * 删除需要移除的信息点
     * @param dbVO
     * @param remove
     * @param BasicfieldSet
     */
    private void remove(ObjectDigital dbVO,Object remove,Set<String> BasicfieldSet){
        if(remove == null){
            return;
        }
        JSONArray array = JSONArray.parseArray(remove.toString());
        array.forEach(item->{
            String removeCode = (String) item;
            if(BasicfieldSet.contains(removeCode)){
                //基础信息
                if("name".equals(removeCode)){
                    dbVO.setName(null);
                }
                if("localId".equals(removeCode)){
                    dbVO.setLocalId(null);
                }
                if("localName".equals(removeCode)){
                    dbVO.setLocalName(null);
                }
                if("virtualCodes".equals(removeCode)){
                    dbVO.setVirtualCodes(new ArrayList<>());
                }
                return;
            }
            dbVO.getInfos().remove(removeCode);
        });
    }

    /**
     * 修改时填充classCode
     * @param voList
     * @param dbMap
     */
    private void fillClassCodeForUpdate(List<ObjectDigital> voList,Map<String, ObjectDigital> dbMap){
        if(voList == null || voList.size()<=0){
            return;
        }
        for (ObjectDigital digital : voList) {
            ObjectDigital dbDigital = dbMap.get(digital.getId());
            if(dbDigital!=null){
                digital.setClassCode(dbDigital.getClassCode());
            }
        }
    }


    /**
     * 查询虚拟信息点
     * @param projectId
     * @param groupCode
     * @param ids
     * @return
     */
    @Override
    public Map<String, List<String>> queryVirtualCodeMap(String projectId,String groupCode,List<String> ids) {
        LambdaQueryWrapper<ObjectDigital> wrapper = new LambdaQueryWrapper<>();
        wrapper.select(ObjectDigital::getId, ObjectDigital::getVirtualCodes);
        wrapper.eq(ObjectDigital::getProjectId,projectId);
        wrapper.eq(ObjectDigital::getGroupCode,groupCode);
        wrapper.in(ObjectDigital::getId,ids);
        wrapper.eq(ObjectDigital::getValid, ValidEnum.TRUE.getType());
        List<ObjectDigital> objectDigitals = queryByCondition(wrapper);
        return CollUtil.fieldValueAsMap(objectDigitals, "id", "virtual_codes");
    }

}
